#!/usr/bin/python3

import sys, os
import struct
import datetime
import tenjin
from tenjin.helpers import *
import configparser

conf = configparser.RawConfigParser()
conf.read('/config/wireless.ini')
dconf = configparser.RawConfigParser()
dconf.read('/config/wireless_default.ini')

def getopt(cport,opt):
    retval=''
    try:
        retval=conf.get(cport,opt)
        if retval=='':
            retval=dconf.get(cport,opt)
    except:
        retval=dconf.get(cport,opt)
    return retval

'''
    Read from a file where information about the wireless meters/telegrams are stored in.
    The wireless application in PI-900S write the information to the file.
    Each row in the file represent a wireless telegram.
    The first 18 bytes in each telegram contains the wanted informaiton.
      * Primary address: byte 0.
      * Timestamp: byte 1 - 9.
      * RSSI: byte 9.
      * Secondary address: byte 10 - 17
    
    Example:
    0643192C5800000000833A44A7327824006807047A8C1000202F2F0C06000000000C14000000000B3B0000000B2D0000000B5A3202000B5E300200046D2008102B0F0000015AEC
    --________________--________________
    
    06               = Primary address 6.
    43192C5800000000 = 64-bit timestamp: 2016-11-16 09:30:59.
    83               = RSSI -125.
    3A44A73278240068 = Secondary address 68002478.32A7.44.3A. Read backwards.
'''

# List of device types.
DeviceTypes = {
    # All missing values are reserved.
    0x00:"Other", 0x01:"Oil", 0x02:"Electricity", 0x03:"Gas",
    0x04:"Heat", 0x05:"Steam", 0x06:"Warm water", 0x07:"Water",
    0x08:"Heat cost allocator", 0x09:"Compressed Air", 0x0A:"Cooling (outlet)", 0x0B:"Cooling (inlet)",
    0x0C:"Heat (inlet)", 0x0D:"Heat/cooling", 0x0E:"Bus/System", 0x0F:"Unknown",
    0x14:"Calorific value", 0x15:"Hot water", 0x16:"Cold water", 0x17:"Dual water",
    0x18:"Pressure", 0x19:"A/D converter", 0x1A:"Smoke detector", 0x1B:"Room sensor",
    0x1C:"Gas detector", 0x20:"Breaker", 0x21:"Valve", 0x25:"Customer unit",
    0x28:"Waste water", 0x29:"Garbage", 0x31:"Communication controller", 0x32:"Unidirectional repeater",
    0x33:"Bidirectional repeater", 0x36:"Radio converter (system)", 0x37:"Radio converter (meter)"
}

def printData(data):
    tstr = ""
    for j in range(len(data)):
        if j&0x0f == 0 and j != 0:
            tstr += '\n'
        tstr += "%02X "%data[j]
    print(tstr)

def getSecondaryAddress(data): # From the wireless telegram
    i_meter_id = struct.unpack("<I",data[4:8])
    i_mfct = struct.unpack("<H",data[2:4])[0]
    meter_id = "%08X"%i_meter_id
    mfct = "%04X"%i_mfct
    ver = data[8]
    med = data[9]
    name = "%c"%(((i_mfct>>10)&0x1F)+64)+"%c"%(((i_mfct>>5)&0x1F)+64)+"%c"%((i_mfct&0x1F)+64)
    meter_type = DeviceTypes[med] if med in DeviceTypes else "%02X"%med
    return meter_id, mfct, "%02X"%(ver), "%02X"%(med), name, meter_type

def getTimeStamp(data):
    # Timestamp: INT64 to YYYY-MM-DD HH:MM:SS
    tmp = struct.unpack("Q", data[0:8])
    tmp = datetime.datetime.fromtimestamp(tmp[0])
    timestamp = "%04d-%02d-%02d %02d:%02d:%02d"%(tmp.year, tmp.month, tmp.day, tmp.hour, tmp.minute, tmp.second)
    return timestamp

def parseLineToBytes(line):
    data = b""
    # Iterate over every other characters.
    for i in range(int(len(line)/2)):
        # Try to convert hex string to int.
        try:
            temp = int(line[2*i:2*i+2], 16)
            data += struct.pack("B",temp)
        except:
            print("Error: '%s' contains a none hexadecimal number."%(line[i]))
            sys.exit()
    return data

#0108FD3F58000000003B
#4644B4091128101505077A54000610EDD653CE9F3D0F1D0E00000000193910011216FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFF000CAA
#        Secondary address:      15102811.09B4.05.07 [BMT] [Water]

# Parse information in 
def parseTelegram(telegram):
    primaryAddress = telegram[0]
    timestamp = getTimeStamp(telegram[1:9])
    (rssi0, rssi1, rssi2, rssi3, rssi4) = struct.unpack('BBBBB', telegram[9:14])
    if (rssi0  & 0xFC) == 0xFC:
        rssi0 = "N/A"
        hc0   = "#"
    else:
        hc0 = rssi0 & 0x03
        rssi0 = -int((rssi0 & 0xFC)/2)
    if (rssi1  & 0xFC) == 0xFC:
        rssi1 = "N/A"
        hc1   = "#"
    else:
        hc1 = rssi1 & 0x03
        rssi1 = -int((rssi1 & 0xFC)/2)
    active     = (rssi4 & 0x02) >> 1
    available  = (rssi4 & 0x04) >> 2
    meter_id, manufacturer, version, medium, name, meter_type = getSecondaryAddress(telegram[15:])
    return ("%s.%s.%s.%s"%(meter_id, manufacturer, version, medium), primaryAddress, name, meter_type, rssi0, rssi1, active, available, timestamp)
    #return ("%s.%s.%s.%s"%(meter_id, manufacturer, version, medium), primaryAddress, name, meter_type, rssi0, timestamp)

def getMyPrimaryAddress(telegram):
	return telegram[14]

# =============================================================================
# === Main ====================================================================
# =============================================================================

fabnum=""
try:
    mbv=os.popen("/binary/wireless -S")
    fabnum=mbv.read().strip()
    mbv.close()
except:
    pass

meterlist = []
primaryaddress = getopt('Main','primaryaddress')
wlmodes=["S1","T1","Invalid mode","S1","R","Invalid mode","Invalid mode","Invalid mode","Invalid mode","Invalid mode","Invalid mode","T1+C"]
n=int(getopt('Main','wirelessmode'))
wirelessmode="Invalid mode"
if n >=0 and n < 12:
    wirelessmode = wlmodes[n]

n_active = 0
n_available = 0
n_active_and_available = 0
try:
    filePath = "/tmp/wireless_telegrams.txt"
    fil = open(filePath)
    for line in fil:
        telegram = parseLineToBytes(line)
        parsed_telegram=parseTelegram(telegram)
        if parsed_telegram[6]:
            n_active += 1
        if parsed_telegram[7]:
            n_available += 1
            if parsed_telegram[6]:
                n_active_and_available += 1
        meterlist.append(parsed_telegram)
        primaryaddress = getMyPrimaryAddress(telegram)
    fil.close()
except:
    pass


tenjin.Engine.cache = tenjin.MemoryCacheStorage()
engine = tenjin.Engine()
# render template with context data
context = { 'title': 'List of Wireless Meters',
           'header': ["Secondary Address", "Prim Addr", "MFCT", "Meter Type", "RSSI 0", "RSSI 1", "Active", "Available", "Time Stamp (LocalTime)"],
            'items': meterlist }
#context = { 'title': 'List of Wireless Meters',
#            'header': ["Secondary Address", "Prim Addr", "MFCT", "Meter Type", "RSSI", "Time Stamp (UTC)"],
#            'items': meterlist }
output = engine.render('templates/wirelessMeterList.pyhtml', context)
print(output)
